package org.scalatest.inspectors.all

import org.scalatest._
import org.scalatest.matchers.{BePropertyMatcher, BePropertyMatchResult, HavePropertyMatcher, HavePropertyMatchResult}
import collection.GenTraversable
import collection.GenMap
class InspectorShorthandsForAllFailedSpec5 extends Spec with Matchers with SharedHelpers {
  class EmptyBePropertyMatcher extends BePropertyMatcher[String] {
    def apply(left: String) = BePropertyMatchResult(left.isEmpty, "empty")
  }
  class StringLengthMatcher(expectedValue: Int) extends HavePropertyMatcher[String, Int] {
    def apply(value: String) = {
      new HavePropertyMatchResult(value.length == expectedValue, "length", expectedValue, value.length)
    }
  }
  val empty = new EmptyBePropertyMatcher()
  def plength(expectedValue: Int) = new StringLengthMatcher(expectedValue)
  val theInstance = "2"
  def arrayToString(xs: GenTraversable[_]): String = xs.toString
  def checkErrorAndCause(e: exceptions.TestFailedException, assertLineNumber: Int, fileName: String, errorMessage: String, causeErrorMessage: String) {
    assert(e.failedCodeFileName == Some(fileName), e.failedCodeFileName + " did not equal " + Some(fileName))
    assert(e.failedCodeLineNumber == Some(assertLineNumber), e.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
    assert(e.message == Some(errorMessage), e.message + " did not equal " + Some(
  errorMessage))
    e.getCause match {
      case tfe: exceptions.TestFailedException =>
        assert(tfe.failedCodeFileName == Some(fileName), tfe.failedCodeFileName + " did not equal " + Some(fileName))
        assert(tfe.failedCodeLineNumber == Some(assertLineNumber), tfe.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
        assert(tfe.message == Some(causeErrorMessage), tfe.message + " did not equal " + Some(causeErrorMessage))
        assert(tfe.getCause == null, tfe.getCause + " was not null")
      case other => fail("Expected cause to be TestFailedException, but got: " + other)
    }
  }
  def checkError(e: exceptions.TestFailedException, assertLineNumber: Int, fileName: String, errorMessage: String) {
    assert(e.failedCodeFileName == Some(fileName), e.failedCodeFileName + " did not equal " + Some(fileName))
    assert(e.failedCodeLineNumber == Some(assertLineNumber), e.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
    assert(e.message == Some(errorMessage), e.message + " did not equal " + Some(
  errorMessage))
  }
  def equal[T](left: T, right: T): Boolean = left == right
  def javaArrayList[T](valueList: List[T]): java.util.ArrayList[T] = {
    val col = new java.util.ArrayList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaHashSet[T](valueList: List[T]): java.util.HashSet[T] = {
    val col = new java.util.HashSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedList[T](valueList: List[T]): java.util.LinkedList[T] = {
    val col = new java.util.LinkedList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaStack[T](valueList: List[T]): java.util.Stack[T] = {
    val col = new java.util.Stack[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaTreeSet[T](valueList: List[T]): java.util.TreeSet[T] = {
    val col = new java.util.TreeSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaVector[T](valueList: List[T]): java.util.Vector[T] = {
    val col = new java.util.Vector[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaArrayBlockingQueue[T](valueList: List[T], size: Int): java.util.concurrent.ArrayBlockingQueue[T] = {
    val col = new java.util.concurrent.ArrayBlockingQueue[T](size)
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaArrayDeque[T](valueList: List[T]): java.util.ArrayDeque[T] = {
    val col = new java.util.ArrayDeque[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaConcurrentLinkedQueue[T](valueList: List[T]): java.util.concurrent.ConcurrentLinkedQueue[T] = {
    val col = new java.util.concurrent.ConcurrentLinkedQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaConcurrentSkipListSet[T](valueList: List[T]): java.util.concurrent.ConcurrentSkipListSet[T] = {
    val col = new java.util.concurrent.ConcurrentSkipListSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaCopyOnWriteArrayList[T](valueList: List[T]): java.util.concurrent.CopyOnWriteArrayList[T] = {
    val col = new java.util.concurrent.CopyOnWriteArrayList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaCopyOnWriteArraySet[T](valueList: List[T]): java.util.concurrent.CopyOnWriteArraySet[T] = {
    val col = new java.util.concurrent.CopyOnWriteArraySet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedBlockingDeque[T](valueList: List[T]): java.util.concurrent.LinkedBlockingDeque[T] = {
    val col = new java.util.concurrent.LinkedBlockingDeque[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedBlockingQueue[T](valueList: List[T]): java.util.concurrent.LinkedBlockingQueue[T] = {
    val col = new java.util.concurrent.LinkedBlockingQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedHashSet[T](valueList: List[T]): java.util.LinkedHashSet[T] = {
    val col = new java.util.LinkedHashSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaPriorityBlockingQueue[T](valueList: List[T]): java.util.concurrent.PriorityBlockingQueue[T] = {
    val col = new java.util.concurrent.PriorityBlockingQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaPriorityQueue[T](valueList: List[T]): java.util.PriorityQueue[T] = {
    val col = new java.util.PriorityQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaHashMap[K, V](valueMap: Map[K, V]): java.util.HashMap[K, V] = {
    val map = new java.util.HashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaTreeMap[K, V](valueMap: Map[K, V]): java.util.TreeMap[K, V] = {
    val map = new java.util.TreeMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaHashtable[K, V](valueMap: Map[K, V]): java.util.Hashtable[K, V] = {
    val map = new java.util.Hashtable[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaConcurrentHashMap[K, V](valueMap: Map[K, V]): java.util.concurrent.ConcurrentHashMap[K, V] = {
    val map = new java.util.concurrent.ConcurrentHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaConcurrentSkipListMap[K, V](valueMap: Map[K, V]): java.util.concurrent.ConcurrentSkipListMap[K, V] = {
    val map = new java.util.concurrent.ConcurrentSkipListMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaLinkedHashMap[K, V](valueMap: Map[K, V]): java.util.LinkedHashMap[K, V] = {
    val map = new java.util.LinkedHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaWeakHashMap[K, V](valueMap: Map[K, V]): java.util.WeakHashMap[K, V] = {
    val map = new java.util.WeakHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaProperties(valueMap: Map[String, String]): java.util.Properties = {
    val map = new java.util.Properties()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Array(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Vector(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaVector(List("hi")), javaVector(List("boom!")), javaVector(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Array(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Vector(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayBlockingQueue(List("hi"), 3), javaArrayBlockingQueue(List("boom!"), 3), javaArrayBlockingQueue(List.empty[String], 3)).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Array(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Vector(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Set(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String]))).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.Seq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = collection.mutable.IndexedSeq(javaArrayDeque(List("hi")), javaArrayDeque(List("boom!")), javaArrayDeque(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Array(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + arrayToString(xs), "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = IndexedSeq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])) should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Vector(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String]))
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = Set(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should contain' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should contain ("hi")
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotContain[String](xs, "hi")) + ", " + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotContain[String](xs, "hi") + " did not contain element \"hi\"")
  }
  def `List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not contain' failed` {
    val xs = List(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not contain "hi"
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColContain[String](xs, "hi")) + ", " + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\" (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColContain[String](xs, "hi") + " contained element \"hi\"")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should be symbol' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColNotIsEmpty[String](xs, 0)) + ", " + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColNotIsEmpty[String](xs, 0) + " was not empty")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not be symbol' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColIsEmpty[String](xs, 0)) + ", " + getFirstJavaColIsEmpty[String](xs, 0) + " was empty (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColIsEmpty[String](xs, 0) + " was empty")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have size' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had size " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected size 0")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have size' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had size 1")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should have length' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeNotEqual[String](xs, 0)) + ", " + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeNotEqual[String](xs, 0) + " had length " + getFirstJavaColSizeNotEqual[String](xs, 0).size + " instead of expected length 0")
  }
  def `Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par should throw TestFailedException with correct stack depth and message when 'java collection should not have length' failed` {
    val xs = Seq(javaConcurrentLinkedQueue(List("hi")), javaConcurrentLinkedQueue(List("boom!")), javaConcurrentLinkedQueue(List.empty[String])).par
    val e = intercept[exceptions.TestFailedException] {
      all(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkErrorAndCause(e, assertLineNumber, "InspectorShorthandsForAllFailedSpec5.scala", "'all' inspection failed, because: \n" + 
  "  at index " + getIndex(xs, getFirstJavaColSizeEqual[String](xs, 1)) + ", " + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1 (InspectorShorthandsForAllFailedSpec5.scala:" + assertLineNumber + ") \n" + 
  "in " + xs, "" + getFirstJavaColSizeEqual[String](xs, 1) + " had length 1")
  }
}
