package org.scalatest.inspectors.every

import org.scalatest._
import org.scalatest.matchers.{BePropertyMatcher, BePropertyMatchResult, HavePropertyMatcher, HavePropertyMatchResult}
import collection.GenTraversable
import collection.GenMap
class InspectorShorthandsForEveryFailedSpec10 extends Spec with Matchers with SharedHelpers {
  class EmptyBePropertyMatcher extends BePropertyMatcher[String] {
    def apply(left: String) = BePropertyMatchResult(left.isEmpty, "empty")
  }
  class StringLengthMatcher(expectedValue: Int) extends HavePropertyMatcher[String, Int] {
    def apply(value: String) = {
      new HavePropertyMatchResult(value.length == expectedValue, "length", expectedValue, value.length)
    }
  }
  val empty = new EmptyBePropertyMatcher()
  def plength(expectedValue: Int) = new StringLengthMatcher(expectedValue)
  val theInstance = "2"
  def arrayToString(xs: GenTraversable[_]): String = xs.toString
  def checkErrorAndCause(e: exceptions.TestFailedException, assertLineNumber: Int, fileName: String, errorMessage: String, causeErrorMessage: String) {
    assert(e.failedCodeFileName == Some(fileName), e.failedCodeFileName + " did not equal " + Some(fileName))
    assert(e.failedCodeLineNumber == Some(assertLineNumber), e.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
    assert(e.message == Some(errorMessage), e.message + " did not equal " + Some(
  errorMessage))
    e.getCause match {
      case tfe: exceptions.TestFailedException =>
        assert(tfe.failedCodeFileName == Some(fileName), tfe.failedCodeFileName + " did not equal " + Some(fileName))
        assert(tfe.failedCodeLineNumber == Some(assertLineNumber), tfe.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
        assert(tfe.message == Some(causeErrorMessage), tfe.message + " did not equal " + Some(causeErrorMessage))
        assert(tfe.getCause == null, tfe.getCause + " was not null")
      case other => fail("Expected cause to be TestFailedException, but got: " + other)
    }
  }
  def checkError(e: exceptions.TestFailedException, assertLineNumber: Int, fileName: String, errorMessage: String) {
    assert(e.failedCodeFileName == Some(fileName), e.failedCodeFileName + " did not equal " + Some(fileName))
    assert(e.failedCodeLineNumber == Some(assertLineNumber), e.failedCodeLineNumber + " did not equal " + Some(assertLineNumber))
    assert(e.message == Some(errorMessage), e.message + " did not equal " + Some(
  errorMessage))
  }
  def equal[T](left: T, right: T): Boolean = left == right
  def javaArrayList[T](valueList: List[T]): java.util.ArrayList[T] = {
    val col = new java.util.ArrayList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaHashSet[T](valueList: List[T]): java.util.HashSet[T] = {
    val col = new java.util.HashSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedList[T](valueList: List[T]): java.util.LinkedList[T] = {
    val col = new java.util.LinkedList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaStack[T](valueList: List[T]): java.util.Stack[T] = {
    val col = new java.util.Stack[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaTreeSet[T](valueList: List[T]): java.util.TreeSet[T] = {
    val col = new java.util.TreeSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaVector[T](valueList: List[T]): java.util.Vector[T] = {
    val col = new java.util.Vector[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaArrayBlockingQueue[T](valueList: List[T], size: Int): java.util.concurrent.ArrayBlockingQueue[T] = {
    val col = new java.util.concurrent.ArrayBlockingQueue[T](size)
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaArrayDeque[T](valueList: List[T]): java.util.ArrayDeque[T] = {
    val col = new java.util.ArrayDeque[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaConcurrentLinkedQueue[T](valueList: List[T]): java.util.concurrent.ConcurrentLinkedQueue[T] = {
    val col = new java.util.concurrent.ConcurrentLinkedQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaConcurrentSkipListSet[T](valueList: List[T]): java.util.concurrent.ConcurrentSkipListSet[T] = {
    val col = new java.util.concurrent.ConcurrentSkipListSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaCopyOnWriteArrayList[T](valueList: List[T]): java.util.concurrent.CopyOnWriteArrayList[T] = {
    val col = new java.util.concurrent.CopyOnWriteArrayList[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaCopyOnWriteArraySet[T](valueList: List[T]): java.util.concurrent.CopyOnWriteArraySet[T] = {
    val col = new java.util.concurrent.CopyOnWriteArraySet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedBlockingDeque[T](valueList: List[T]): java.util.concurrent.LinkedBlockingDeque[T] = {
    val col = new java.util.concurrent.LinkedBlockingDeque[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedBlockingQueue[T](valueList: List[T]): java.util.concurrent.LinkedBlockingQueue[T] = {
    val col = new java.util.concurrent.LinkedBlockingQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaLinkedHashSet[T](valueList: List[T]): java.util.LinkedHashSet[T] = {
    val col = new java.util.LinkedHashSet[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaPriorityBlockingQueue[T](valueList: List[T]): java.util.concurrent.PriorityBlockingQueue[T] = {
    val col = new java.util.concurrent.PriorityBlockingQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaPriorityQueue[T](valueList: List[T]): java.util.PriorityQueue[T] = {
    val col = new java.util.PriorityQueue[T]()
    for (value <- valueList)
      col.add(value)
    col
  }
  def javaHashMap[K, V](valueMap: Map[K, V]): java.util.HashMap[K, V] = {
    val map = new java.util.HashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaTreeMap[K, V](valueMap: Map[K, V]): java.util.TreeMap[K, V] = {
    val map = new java.util.TreeMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaHashtable[K, V](valueMap: Map[K, V]): java.util.Hashtable[K, V] = {
    val map = new java.util.Hashtable[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaConcurrentHashMap[K, V](valueMap: Map[K, V]): java.util.concurrent.ConcurrentHashMap[K, V] = {
    val map = new java.util.concurrent.ConcurrentHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaConcurrentSkipListMap[K, V](valueMap: Map[K, V]): java.util.concurrent.ConcurrentSkipListMap[K, V] = {
    val map = new java.util.concurrent.ConcurrentSkipListMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaLinkedHashMap[K, V](valueMap: Map[K, V]): java.util.LinkedHashMap[K, V] = {
    val map = new java.util.LinkedHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaWeakHashMap[K, V](valueMap: Map[K, V]): java.util.WeakHashMap[K, V] = {
    val map = new java.util.WeakHashMap[K, V]()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def javaProperties(valueMap: Map[String, String]): java.util.Properties = {
    val map = new java.util.Properties()
    for ((key, value) <- valueMap)
      map.put(key, value)
    map
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Vector(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaConcurrentSkipListMap(Map.empty[String, String]), javaConcurrentSkipListMap(Map("b" -> "boom!")), javaConcurrentSkipListMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Array(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Vector(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaLinkedHashMap(Map.empty[String, String]), javaLinkedHashMap(Map("b" -> "boom!")), javaLinkedHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Array(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + arrayToString(xs))
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Vector(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))) should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Set(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `(new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = (new collection.mutable.ListBuffer() ++ List(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!")))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.Seq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should be ('empty)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was not empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not be symbol' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not be 'empty
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} was empty (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapIsEmpty[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have size 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size {2} instead of expected size 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have size' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have size (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had size 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should have length 0
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length {2} instead of expected length 0 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeNotEqual[String, String](itr, xs, 0)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not have length' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not have length (1)
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} had length 1 (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapSizeEqual[String, String](itr, xs, 1)) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain key "b"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain key' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain key ("b")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained key \"b\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainKey[String, String](itr, xs, "b")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should contain value "boom!"
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + ", \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} did not contain value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapNotContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
  def `collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par should throw TestFailedException with correct stack depth and message when 'java map should not contain value' failed` {
    val xs = collection.mutable.IndexedSeq(javaWeakHashMap(Map.empty[String, String]), javaWeakHashMap(Map("b" -> "boom!")), javaWeakHashMap(Map("h" -> "hello!"))).par
    val itr = xs.toIterator
    val e = intercept[exceptions.TestFailedException] {
      every(xs) should not contain value ("boom!")
    }
    val assertLineNumber = thisLineNumber - 2
    checkError(e, assertLineNumber, "InspectorShorthandsForEveryFailedSpec10.scala", "'every' inspection failed, because: \n" + 
  "  " + new java.text.MessageFormat("at index {0}, {1} contained value \"boom!\" (InspectorShorthandsForEveryFailedSpec10.scala:" + assertLineNumber + ")").format(indexElementJavaMapContainValue[String, String](itr, xs, "boom!")) + " \n" + 
  "in " + xs)
  }
}
